#include <stdio.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/types.h>

int main(int argc, char **argv) {

	DIR *dp = NULL;
	struct dirent *dirinfo;
	struct stat inodeinfo;
	char buf[200];

    if (!(dp = opendir(argv[1]))) {
        fprintf(stderr, "CANNOT OPEN DIR %s\n", argv[1]);
        return -1;
    }

	/* Dateiname und INodeNr. ausgeben */
	printf("\n");
	printf("            DirInfos      |               INodeInfos\n\n");
	printf("      Dateiname   INodenr | Dateigroesse  Zeitstempel    Dateityp\n\n");
	while ((dirinfo = readdir(dp)) != NULL) {
		printf("%15s", dirinfo->d_name);
		/* Achtung: Variblen wie d_ino koennen auf unterschiedlichen */
	        /* Rechnern unterschiedlich definiert sein (typedef). Um */
                /* herauszufinden, welcher Datentyp diesen Variablen zugrunde *
                /* liegt, entsprechende Header-Datei anschauen... */		                        
		printf("%10li |", dirinfo->d_ino);

		/* INode-Informationen ausgeben. */
		/* dirinfo->d_name gibt nur die Dateinamen aus und nicht den */
		/* zugehrigen Pfad. Fehlt die Pfadangabe such lstat nur im */
		/* aktuellen Verzeichnis nach den entsprechenden Dateien und */
		/* INodeInformationen. */
		sprintf(buf, "%s/%s", argv[1], dirinfo->d_name);
        if (lstat(buf, &inodeinfo) < 0) {
            fprintf(stderr, "CANNOT READ INFO OF %s\n", dirinfo->d_name);
        }
		else {
			/* Dateigroesse */
			printf("%8li", inodeinfo.st_size);
			/* Zeitstempel */
			printf("%17li", inodeinfo.st_mtime);
			/* Dateityp */
			printf("%10lx", inodeinfo.st_mode);

			/* Die Konstante S_IFDIR (0040000) steht fr Verzeichnisse, */
			/* die Konstante S_IFREG (0100000) steht fr Dateien. */

			/* Wird inodeinfo.st_mode mit der Konstanten S_IFMT */
			/* (0160000) bitweise verknpft (&), dann erhlt man */ 
			/* als Ergebnis die Unterscheidung Datei oder Verzeichnis. */
			/* Ohne diese Verknpfung wird noch zwischen den */
			/* verschiedenen Dateiarten wie z.B. ausfhrbar */
			/* etc. unterschieden! */

			if ((inodeinfo.st_mode&S_IFMT) == S_IFDIR) {
				printf(" (Verz.)");
			}
			if ((inodeinfo.st_mode&S_IFMT) == S_IFREG) {
				printf(" (Datei)");
			}
		}
		printf("\n");
	}

	/* Closedir nicht vergessen :-) */
	closedir(dp);

	printf("\n");
	return 0;

}
